@@ -47,60 +47,66 @@ Akonadi::NotificationCollector::~NotificationCollector()
 
 void Akonadi::NotificationCollector::itemAdded( const PimItem &item,
                                                 const Collection &collection,
-                                                const QString & mimeType,
                                                 const QByteArray & resource )
 {
-  itemNotification( NotificationMessage::Add, item, collection, Collection(), mimeType, resource );
+  itemNotification( NotificationMessageV2::Add, item, collection, Collection(), resource );
 }
 
 void Akonadi::NotificationCollector::itemChanged( const PimItem &item,
                                                   const QSet<QByteArray> &changedParts,
                                                   const Collection &collection,
-                                                  const QString & mimeType,
                                                   const QByteArray & resource )
 {
-  itemNotification( NotificationMessage::Modify, item, collection, Collection(), mimeType, resource, changedParts );
+  itemNotification( NotificationMessageV2::Modify, item, collection, Collection(), resource, changedParts );
 }
 
-void Akonadi::NotificationCollector::itemMoved( const PimItem &item,
-                                                const Collection &collectionSrc,
-                                                const Collection &collectionDest,
-                                                const QString &mimeType,
-                                                const QByteArray &sourceResource )
+void Akonadi::NotificationCollector::itemsFlagsChanged( const PimItem::List &items,
+                                                        const QSet< QByteArray > &addedFlags,
+                                                        const QSet< QByteArray > &removedFlags,
+                                                        const Collection &collection,
+                                                        const QByteArray &resource)
 {
-  itemNotification( NotificationMessage::Move, item, collectionSrc, collectionDest, mimeType, sourceResource );
+  itemNotification( NotificationMessageV2::ModifyFlags, items, collection, Collection(), resource, QSet<QByteArray>(), addedFlags, removedFlags );
 }
 
 
-void Akonadi::NotificationCollector::itemRemoved( const PimItem &item,
-                                                  const Collection &collection,
-                                                  const QString & mimeType,
-                                                  const QByteArray & resource )
+void Akonadi::NotificationCollector::itemsMoved( const PimItem::List &items,
+                                                 const Collection &collectionSrc,
+                                                 const Collection &collectionDest,
+                                                 const QByteArray &sourceResource )
 {
-  itemNotification( NotificationMessage::Remove, item, collection, Collection(), mimeType, resource );
+  itemNotification( NotificationMessageV2::Move, items, collectionSrc, collectionDest, sourceResource );
 }
 
-void NotificationCollector::itemLinked(const PimItem & item, const Collection & collection)
+
+void Akonadi::NotificationCollector::itemsRemoved( const PimItem::List &items,
+                                                   const Collection &collection,
+                                                   const QByteArray & resource )
 {
-  itemNotification( NotificationMessage::Link, item, collection, Collection(), QString(), QByteArray() );
+  itemNotification( NotificationMessageV2::Remove, items, collection, Collection(), resource );
 }
 
-void NotificationCollector::itemUnlinked(const PimItem & item, const Collection & collection)
+void NotificationCollector::itemsLinked(const PimItem::List & items, const Collection & collection)
 {
-  itemNotification( NotificationMessage::Unlink, item, collection, Collection(), QString(), QByteArray() );
+  itemNotification( NotificationMessageV2::Link, items, collection, Collection(), QByteArray() );
+}
+
+void NotificationCollector::itemsUnlinked(const PimItem::List & items, const Collection & collection)
+{
+  itemNotification( NotificationMessageV2::Unlink, items, collection, Collection(), QByteArray() );
 }
 
 void Akonadi::NotificationCollector::collectionAdded( const Collection &collection,
                                                       const QByteArray &resource )
 {
-  collectionNotification( NotificationMessage::Add, collection, collection.parentId(), -1, resource );
+  collectionNotification( NotificationMessageV2::Add, collection, collection.parentId(), -1, resource );
 }
 
 void Akonadi::NotificationCollector::collectionChanged( const Collection &collection,
                                                         const QList<QByteArray> &changes,
                                                         const QByteArray &resource )
 {
-  collectionNotification( NotificationMessage::Modify, collection, collection.parentId(), -1, resource, changes.toSet() );
+  collectionNotification( NotificationMessageV2::Modify, collection, collection.parentId(), -1, resource, changes.toSet() );
 }
 
 void NotificationCollector::collectionMoved( const Collection &collection,
@@ -108,25 +114,25 @@ void NotificationCollector::collectionMoved( const Collection &collection,
                                              const QByteArray &resource,
                                              const QByteArray &destResource )
 {
-  collectionNotification( NotificationMessage::Move, collection, source.id(), collection.parentId(), resource, QSet<QByteArray>(), destResource );
+  collectionNotification( NotificationMessageV2::Move, collection, source.id(), collection.parentId(), resource, QSet<QByteArray>(), destResource );
 }
 
 void Akonadi::NotificationCollector::collectionRemoved( const Collection &collection,
                                                         const QByteArray &resource )
 {
-  collectionNotification( NotificationMessage::Remove, collection, collection.parentId(), -1, resource );
+  collectionNotification( NotificationMessageV2::Remove, collection, collection.parentId(), -1, resource );
 }
 
 void NotificationCollector::collectionSubscribed( const Collection& collection,
                                                   const QByteArray& resource )
 {
-  collectionNotification( NotificationMessage::Subscribe, collection, collection.parentId(), -1, resource, QSet<QByteArray>() );
+  collectionNotification( NotificationMessageV2::Subscribe, collection, collection.parentId(), -1, resource, QSet<QByteArray>() );
 }
 
 void NotificationCollector::collectionUnsubscribed( const Collection& collection,
                                                     const QByteArray& resource )
 {
-  collectionNotification( NotificationMessage::Unsubscribe, collection, collection.parentId(), -1, resource, QSet<QByteArray>() );
+  collectionNotification( NotificationMessageV2::Unsubscribe, collection, collection.parentId(), -1, resource, QSet<QByteArray>() );
 }
 
 
@@ -150,72 +156,94 @@ void NotificationCollector::setSessionId(const QByteArray &sessionId)
   mSessionId = sessionId;
 }
 
-void NotificationCollector::itemNotification( NotificationMessage::Operation op,
+void NotificationCollector::itemNotification( NotificationMessageV2::Operation op,
                                               const PimItem & item,
                                               const Collection & collection,
                                               const Collection & collectionDest,
-                                              const QString & mimeType,
                                               const QByteArray & resource,
                                               const QSet<QByteArray> &parts )
 {
-  QVector<Collection> collections;
+  PimItem::List items;
+  items << item;
+  itemNotification( op, items, collection, collectionDest, resource, parts );
+}
 
-  if ( op == NotificationMessage::Modify ) {
-    collections = DataStore::self()->virtualCollections(item);
+void NotificationCollector::itemNotification( NotificationMessageV2::Operation op,
+                                              const PimItem::List & items,
+                                              const Collection & collection,
+                                              const Collection & collectionDest,
+                                              const QByteArray & resource,
+                                              const QSet<QByteArray> &parts,
+                                              const QSet<QByteArray> &addedFlags,
+                                              const QSet<QByteArray> &removedFlags )
+{
+  Collection notificationDestCollection;
+  QMap<Entity::Id, PimItem> vCollections;
+
+  if (( op == NotificationMessageV2::Modify ) ||
+      ( op == NotificationMessageV2::ModifyFlags )) {
+    vCollections = DataStore::self()->virtualCollections( items );
   }
 
   if ( !collection.isValid() ) {
-    collections.append( item.collection() );
+    notificationDestCollection = items.first().collection();
   } else {
-    collections.append ( collection );
+    notificationDestCollection = collection;
   }
 
-  QByteArray destResourceName;
+  NotificationMessageV2 msg;
+  msg.setSessionId( mSessionId );
+  msg.setType( NotificationMessageV2::Items );
+  msg.setOperation( op );
+
+  msg.setItemParts( parts );
+  msg.setAddedFlags( addedFlags );
+  msg.setRemovedFlags( removedFlags );
+
   if ( collectionDest.isValid() ) {
+    QByteArray destResourceName;
     destResourceName = collectionDest.resource().name().toLatin1();
+    msg.setDestinationResource( destResourceName );
   }
 
-  QString mt = mimeType;
-  if ( mt.isEmpty() ) {
-    mt = item.mimeType().name();
+  msg.setParentDestCollection( collectionDest.id() );
+
+  /* Notify all virtual collections the items are linked to. */
+  Q_FOREACH ( const Entity::Id &colId, vCollections.uniqueKeys() ) {
+    NotificationMessageV2 copy = msg;
+
+    Q_FOREACH ( const PimItem &item, vCollections.values( colId ) ) {
+      copy.addEntity( item.id(), item.remoteId(), item.remoteRevision(), item.mimeType().name() );
+    }
+    copy.setParentCollection( colId );
+    copy.setResource( resource );
+
+    dispatchNotification( copy );
   }
 
-  QSet<QByteArray> itemParts;
-  itemParts << item.remoteRevision().toUtf8();
-
-  /* Notify parent collection as well as all virtual collections the item
-   * is linked to */
-  Q_FOREACH( const Collection &col, collections ) {
-    NotificationMessage msg;
-    msg.setSessionId( mSessionId );
-    msg.setType( NotificationMessage::Item );
-    msg.setOperation( op );
-    msg.setUid( item.id() );
-    msg.setRemoteId( item.remoteId() );
-    msg.setItemParts( parts );
-
-    //HACK: store remoteRevision in itemparts for deletion
-    if ( op == NotificationMessage::Remove )
-      msg.setItemParts( itemParts );
-
-    if ( !destResourceName.isEmpty() ) // only relevant for moves
-      msg.setDestinationResource( destResourceName );
-
-    msg.setParentCollection( col.id() );
-    // will be valid if it is a move message
-    msg.setParentDestCollection( collectionDest.id() );
-    msg.setMimeType( mt );
-
-    QByteArray res = resource;
-    if ( res.isEmpty() )
-      res = col.resource().name().toLatin1();
-    msg.setResource( res );
-
-    dispatchNotification( msg );
+  Q_FOREACH ( const PimItem &item, items ) {
+    msg.addEntity( item.id(), item.remoteId(), item.remoteRevision(), item.mimeType().name() );
   }
+
+  Collection col;
+  if ( !collection.isValid() ) {
+    msg.setParentCollection( items.first().collection().id() );
+    col = items.first().collection();
+  } else {
+    msg.setParentCollection( collection.id() );
+    col = collection;
+  }
+
+  QByteArray res = resource;
+  if ( res.isEmpty() ) {
+    res = col.resource().name().toLatin1();
+  }
+  msg.setResource( res );
+
+  dispatchNotification( msg );
 }
 
-void NotificationCollector::collectionNotification( NotificationMessage::Operation op,
+void NotificationCollector::collectionNotification( NotificationMessageV2::Operation op,
                                                     const Collection & collection,
                                                     Collection::Id source,
                                                     Collection::Id destination,
@@ -223,34 +251,30 @@ void NotificationCollector::collectionNotification( NotificationMessage::Operati
                                                     const QSet<QByteArray> &changes,
                                                     const QByteArray & destResource )
 {
-  NotificationMessage msg;
-  msg.setType( NotificationMessage::Collection );
+  NotificationMessageV2 msg;
+  msg.setType( NotificationMessageV2::Collections );
   msg.setOperation( op );
   msg.setSessionId( mSessionId );
-  msg.setUid( collection.id() );
-  msg.setRemoteId( collection.remoteId() );
+  msg.addEntity( collection.id(), collection.remoteId(), collection.remoteRevision() );
   msg.setParentCollection( source );
   msg.setParentDestCollection( destination );
   msg.setDestinationResource( destResource );
   msg.setItemParts( changes );
 
-  //HACK: store remoteRevision in itemparts for deletion
-  if ( op == NotificationMessage::Remove )
-    msg.setItemParts( QSet<QByteArray>() << collection.remoteRevision().toUtf8() );
-
   QByteArray res = resource;
   if ( res.isEmpty() )
     res = collection.resource().name().toLatin1();
   msg.setResource( res );
+
   dispatchNotification( msg );
 }
 
-void NotificationCollector::dispatchNotification(const NotificationMessage & msg)
+void NotificationCollector::dispatchNotification(const NotificationMessageV2 & msg)
 {
   if ( !mDb || mDb->inTransaction() ) {
-    NotificationMessage::appendAndCompress( mNotifications, msg );
+    NotificationMessageV2::appendAndCompress( mNotifications, msg );
   } else {
-    NotificationMessage::List l;
+    NotificationMessageV2::List l;
     l << msg;
     Q_EMIT notify( l );
   }
